#!/usr/bin/env python

import hashlib
import optparse
import os
import sys

import zerorpc

DEBUG = False


def _log(message):
    if not DEBUG:
        return
    sys.stderr.write(message)
    sys.stderr.write("\n")


class NotFound(Exception):
    pass


class StorageRPC(object):
    def __init__(self, storage_dir):
        self.storage_dir = storage_dir

    def store(self, blob):
        blob_id = hashlib.sha1(blob).hexdigest()
        _log("STORE {}".format(blob_id))
        blob_path = self._get_blob_path(blob_id)
        if not os.path.exists(blob_path):
            with self._open(blob_path, 'wb') as f:
                f.write(blob)
        return blob_id

    def retrieve(self, blob_id):
        _log("RETRIEVE {}".format(blob_id))
        blob_path = self._get_blob_path(blob_id)
        try:
            with self._open(blob_path, 'rb') as f:
                return f.read()
        except IOError:
            raise NotFound(
                "Unable to retrieve blob with id: {}".format(blob_id))

    def delete(self, blob_id):
        _log("DELETE {}".format(blob_id))
        try:
            os.remove(self._get_blob_path(blob_id))
        except OSError:
            raise NotFound(
                "Unable to retrieve blob with id: {}".format(blob_id))

    @zerorpc.stream
    def list(self):
        _log("LIST")
        ## ok, now go and figure out the list of stored blobs :P
        for l1dir in os.listdir(self.storage_dir):
            l1dir_path = os.path.join(self.storage_dir, l1dir)
            for blobfile in os.listdir(l1dir_path):
                yield l1dir + blobfile

    def _get_blob_path(self, blob_id):
        return os.path.join(self.storage_dir, blob_id[:2], blob_id[2:])

    def _open(self, name, mode='rb'):
        _basedir = os.path.dirname(name)
        if 'b' not in mode:
            mode += 'b' # We always want binary mode!
        if not os.path.exists(_basedir):
            os.makedirs(_basedir)
        return open(name, mode)


def main():
    from . import DEFAULT_PORT, DEFAULT_ADDR

    parser = optparse.OptionParser(
        usage="%prog [options] <command> [args..]"
    )
    parser.disable_interspersed_args()
    parser.add_option("--storage", action='store', dest='storage_dir',
                      metavar="DIRECTORY",
                      help="Base directory for the blobs storage. If it does"
                           "not exist, it will be created. Required.")
    parser.add_option("--port", action='store', dest='listen_port',
                      metavar="PORT",
                      help="TCP port on which to listen for connections. "
                           "Defaults to {port}".format(port=DEFAULT_PORT))
    parser.add_option("--address", action='store', dest='listen_address',
                      metavar="ADDRESS",
                      help="IP address on which to listen for connections. "
                           "Defaults to {addr}".format(addr=DEFAULT_ADDR))
    parser.add_option("--debug", action='store_true', dest='enable_debug',
                      default=False,
                      help="Whether to enable debugging output.")
    opts, args = parser.parse_args()

    if opts.storage_dir is None:
        sys.stderr.write(
            "You must specify a storage directory using the --storage "
            "argument.\n"
            "See --help for more information.")
        return 1

    listen_port = opts.listen_port or DEFAULT_PORT
    listen_address = opts.listen_address or DEFAULT_ADDR

    s = zerorpc.Server(StorageRPC(opts.storage_dir))
    s.bind("tcp://{}:{}".format(listen_address, listen_port))

    print "Listening on {}:{}".format(listen_address, listen_port)

    try:
        s.run()
    except KeyboardInterrupt:
        print "Server terminated."

    return 0


if __name__ == '__main__':
    sys.exit(main())



## blobstore setup script

from setuptools import setup, find_packages

version = __import__('blobstore').__version__

description = "Generic, zerorpc-powered, blobs storage"

setup(
    name='blobstore',
    version=version,
    url='http://github.com/rshk/blobstore',
    author='Samuele Santi',
    author_email='samuele@samuelesanti.com',
    description=description,
    long_description=description,
    license='GNU General Public License v3 or later (GPLv3+)',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'zerorpc',
    ],
    entry_points = {
        'console_scripts': [
            'blobstore_server = blobstore.server:main',
            'blobstore_client = blobstore.client:main',
            ],
        },
    #classifiers=[],
)



#!/usr/bin/env python

# This is the client; just a wrapper around calls in the server..

import optparse
import sys

import zerorpc


COMMANDS_HELP = """\
store [<filename>]
    Stores the specified file. If no filename is passed or the filename
    is '-', data will be read from standard input.
    Prints the resulting blob_id on the standard output.

retrieve <blob_id> [<filename>]
    Retrieves the object specified by <blob_id> and stores it inside
    <filename>. If no <filename> was specified or <filename> is '-',
    write the object to standard output instead.

delete <blob_id>
    Silently deletes the object with specified id.

list
    Prints a list of stored objects ids.
"""


def main():
    parser = optparse.OptionParser()
    parser.disable_interspersed_args()
    parser.add_option("--port", action='store', dest='server_port')
    parser.add_option("--address", action='store', dest='server_address')
    opts, args = parser.parse_args()

    from . import DEFAULT_PORT, DEFAULT_ADDR
    server_port = opts.server_port or DEFAULT_PORT
    server_address = opts.server_address or DEFAULT_ADDR

    client = zerorpc.Client()
    client.connect("tcp://{}:{}".format(server_address, server_port))

    command = args[0]

    if command == 'help':
        print COMMANDS_HELP

    elif command == 'store':
        try:
            filename = args[1]
        except IndexError:
            filename = None

        if (filename is None) or (filename == '-'):
            blob = sys.stdin.read()
        else:
            with open(filename, 'rb') as f:
                blob = f.read()

        print client.store(blob)

    elif command == 'retrieve':
        try:
            blob_id = args[1]
        except IndexError:
            sys.stderr.write("Usage: retrieve <blob_id> [<filename>]")
            return 1

        try:
            filename = args[2]
        except IndexError:
            filename = None

        if (filename is None) or (filename == '-'):
            sys.stdout.write(client.retrieve(blob_id))
        else:
            with open(filename, 'wb') as f:
                f.write(client.retrieve(blob_id))

    elif command == 'delete':
        try:
            blob_id = args[1]
        except IndexError:
            sys.stderr.write("Usage: delete <blob_id>")
            return 1

        client.delete(blob_id)

    elif command == 'list':
        for blob_id in client.list():
            print blob_id

    else:
        sys.stderr.write(
            "No such command: {command}\n"
            "See 'help' for more information on commands.\n"
            "".format(command=command))
        return 1

    return 0


if __name__ == '__main__':
    sys.exit(main())
